package com.anthonycicchetti.slackbot.osrs

import com.fasterxml.jackson.annotation.JsonIgnoreProperties
import com.fasterxml.jackson.databind.ObjectMapper
import com.fasterxml.jackson.module.kotlin.readValue
import com.github.kittinunf.fuel.Fuel
import com.github.kittinunf.result.Result
import org.slf4j.LoggerFactory
import java.nio.charset.Charset

class ItemMapper {
    companion object {
        private val logger = LoggerFactory.getLogger("ItemMapper")
        private val mapper = ObjectMapper()
        fun getItemByName(itemName: String): ApiItem? =
            itemList.firstOrNull { item -> item.name.lowercase() == itemName.lowercase() }

        fun getItemPrice(item: ApiItem): Item {
            val response = Fuel.get("https://prices.runescape.wiki/api/v1/osrs/latest", listOf("id" to item.id))
                .appendHeader("User-Agent", "private_slackbot - @AnthonyCicc")
                .responseString(Charset.defaultCharset()).third
            return when (response) {
                is Result.Success -> {
                    val r: Resp = mapper.readValue(response.get())
                    r.data.getOrDefault(item.id.toString(), Item())
                }
                is Result.Failure -> {
                    logger.warn("Couldn't find Item with ID ${item.id}")
                    Item()
                }
            }
        }

        private val itemList: List<ApiItem> = run {
            val response = Fuel.get("https://prices.runescape.wiki/api/v1/osrs/mapping")
                .appendHeader("User-Agent", "private_slackbot - @AnthonyCicc")
                .responseString(Charset.defaultCharset()).third
            when (response) {
                is Result.Success -> {
                    logger.info("Got item mappings")
                    mapper.readValue(response.get())
                }
                is Result.Failure -> {
                    logger.info("Could not get item mappings")
                    emptyList()
                }
            }
        }
    }
}

data class Resp(
    val data: Map<String, Item> = emptyMap(),
)

data class Item (
    val low: Int = 0,
    val lowTime: Long = 0L,
    val high: Int = 0,
    val highTime: Long = 0L,
)


@JsonIgnoreProperties(ignoreUnknown = true)
data class ApiItem(
    val examine: String = "Empty Examine Text",
    val id: Int = 0,
    val members: Boolean = false,
    val lowalch: Int = 0,
    val limit: Int = 0,
    val highalch: Int = 0,
    val name: String = "Empty Name",
    val value: Int = 0,
) {
}